<?php
namespace App\DTO\Validation;

use App\DTO\Validation\Rules\ValidationRuleInterface;
use ReflectionAttribute;
use ReflectionClass;

/*
* Get validation attributes from classes and execute validator logic 
*/
class Validator{

  private $errors = [];

  public function validate(object $dto):void
  {

    /**Here we are injecting concret class instead of class name. Passing concret class or 
     * already instantiated object to reflection class helps to extract not only property but also
     * property values.
     */
    $reflector = new ReflectionClass($dto);

    //Get all available class properties from the passed object variable.
    //In this case dto
    foreach($reflector->getProperties() as $property)
    {

      //Filtering and extracting only the PHP Attributes that implements ValidationRuleInterface
      $attributes = $property->getAttributes(
                                ValidationRuleInterface::class,
                                ReflectionAttribute::IS_INSTANCEOF
                              );
      
      /**
       * This return a php class where you define as a #[Attribute]
       * If you are accessing methods or properties of that class make sure to instantiate it.
       */
      foreach($attributes as $attribute)
      {
        //Return value to $validator is a instanciated object of validatorRuleinterface
        $validator = $attribute->newInstance()->getValidator();

        //If validation failed.
        if(!$validator->validate($property->getValue($dto)))
        {
          $attributeName = $attribute->getName();
          $parts = explode('\\', $attributeName);
          $errorName = end($parts);

          $this->errors[$property->getName()][] = sprintf(
                            "%s validation failed",
                            $errorName, 
                            );
        }

      }
    }
  }

  /**Return errors */
  public function getErrors($index=''):array|string
  {
    if($index){
      $errorMessage = isset($this->errors[$index][0])?$this->errors[$index][0]:'';
      return $errorMessage;
    }else{
      return $this->errors;
    }
    
  }
}